/** @file
===========================================================================
MODULE				: CCustomBaseFilter

DESCRIPTIONS		: Custom Base class implementation

REVISION HISTORY	: Version 1.0

COPYRIGHT			: (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS		: The information/data/code contained within this file is 
					  the property of CSIR, Meraka Institute and has been 
					  classified as PUBLIC.
===========================================================================
*/

#include <DirectShow/Common/stdafx.h>
#include "CustomBaseFilter.h"

CCustomBaseFilter::CCustomBaseFilter(TCHAR *pObjectName, LPUNKNOWN lpUnk, CLSID clsid)
: CTransformFilter(pObjectName, lpUnk, clsid),
m_nInHeight(0),
m_nInWidth(0),
m_nInPixels(0),
m_nOutHeight(0),
m_nOutWidth(0),
m_nOutPixels(0),
m_nBitCount(0)
{
}

CCustomBaseFilter::~CCustomBaseFilter(void)
{
}

void CCustomBaseFilter::AddInputType( const GUID* pType, const GUID* pSubType, const GUID* pFormat )
{
	m_vInTypes.push_back((GUID*)pType);
	m_vInSubTypes.push_back((GUID*)pSubType);
	m_vInFormats.push_back((GUID*)pFormat);
}

HRESULT CCustomBaseFilter::CheckInputType( const CMediaType *pMediaType )
{
	for (int i = 0; i < m_vInTypes.size(); i++)
	{
		if (((pMediaType->majortype == *m_vInTypes[i]) && (m_vInTypes[i]))||(!m_vInTypes[i]))
		{
			//Compare sub type
			if (((pMediaType->subtype == *m_vInSubTypes[i]) && (m_vInSubTypes[i]))||(!m_vInSubTypes[i]))
			{
				//Compare formats
				if (((pMediaType->formattype == *m_vInFormats[i])&&(m_vInFormats[i]))||(!m_vInFormats[i]))
				{
					return S_OK;
				}
			}
		}
	}
	return VFW_E_TYPE_NOT_ACCEPTED;
}

HRESULT CCustomBaseFilter::SetMediaType( PIN_DIRECTION direction, const CMediaType *pmt )
{
	if (direction == PINDIR_INPUT)
	{
		if (pmt->formattype == FORMAT_VideoInfo)
		{
			// WARNING! In general you cannot just copy a VIDEOINFOHEADER
			// struct, because the BITMAPINFOHEADER member may be followed by
			// random amounts of palette entries or color masks. (See VIDEOINFO
			// structure in the DShow SDK docs.) Here it's OK because we just
			// want the information that's in the VIDEOINFOHEADER struct itself.

			VIDEOINFOHEADER* pVih = (VIDEOINFOHEADER*) pmt->pbFormat;
			CopyMemory(&m_videoInHeader, pVih, sizeof(VIDEOINFOHEADER));
			//Set these values now so we don't have to repeat it for each transform
			m_nInHeight = abs(m_videoInHeader.bmiHeader.biHeight);
			m_nInWidth = m_videoInHeader.bmiHeader.biWidth;
			m_nInPixels = m_nInHeight * m_nInWidth;
			m_nBitCount = m_videoInHeader.bmiHeader.biBitCount;
		}
	}
	else   // output pin
	{
		ASSERT(direction == PINDIR_OUTPUT);
		if (pmt->formattype == FORMAT_VideoInfo)
		{
			// Store a copy of the video header info
			VIDEOINFOHEADER *pVih = (VIDEOINFOHEADER*)pmt->pbFormat;
			CopyMemory(&m_videoOutHeader, pVih, sizeof(VIDEOINFOHEADER));
			//Set these values now so we don't have to repeat it for each transform
			m_nOutHeight = abs(m_videoOutHeader.bmiHeader.biHeight);
			m_nOutWidth = m_videoOutHeader.bmiHeader.biWidth;
			m_nOutPixels = m_nOutHeight * m_nOutWidth;
		}
	}
	return S_OK;
}

HRESULT CCustomBaseFilter::Transform( IMediaSample *pSource, IMediaSample *pDest )
{
	// Get pointers to the underlying buffers.
	long lSrcDataLen = pSource->GetSize();
	long lDestDataLen = pDest->GetSize();

	BYTE *pBufferIn, *pBufferOut;
	HRESULT hr = pSource->GetPointer(&pBufferIn);
	if (FAILED(hr))
	{
		return hr;
	}
	
	hr = pDest->GetPointer(&pBufferOut);
	if (FAILED(hr))
	{
		return hr;
	}
	// Process the data.

	DWORD cbDest = ApplyTransform(pBufferIn, pBufferOut);
	
	KASSERT((long)cbDest <= pDest->GetSize());

	pDest->SetActualDataLength(cbDest);
	pDest->SetSyncPoint(TRUE);
	return S_OK;
}

STDMETHODIMP CCustomBaseFilter::NonDelegatingQueryInterface( REFIID riid, void **ppv )
{
	if(riid == (IID_ISettingsInterface))
	{
		return GetInterface((ISettingsInterface*) this, ppv);
	}
	else if (riid == IID_IStatusInterface)
	{
		return GetInterface((IStatusInterface*) this, ppv);
	}
	else
	{
		return CTransformFilter::NonDelegatingQueryInterface(riid, ppv);
	}
}

STDMETHODIMP CCustomBaseFilter::GetParameter( const char* szParamName, char* szResult , int& nResultLength)
{
	return S_FALSE;
}

STDMETHODIMP CCustomBaseFilter::SetParameter( const char* szParamName, const char* szValue )
{
	return S_FALSE;
}
