/** @file
===========================================================================
MODULE				: MultiIOBaseFilter

DESCRIPTIONS		: This base class takes care of doing most of the menial work when writing a transform filter that has multiple input and output pins.

REVISION HISTORY	: Version 1.0

COPYRIGHT			: (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS		: The information/data/code contained within this file is 
					  the property of CSIR, Meraka Institute and has been 
					  classified as PUBLIC.
===========================================================================
*/

#include "MultiIOOutputPin.h"

#include "MultiIOBaseFilter.h"
#include "MultiIOInputPin.h"

CMultiIOOutputPin::CMultiIOOutputPin(CMultiIOBaseFilter* pFilter, CCritSec* pLock, HRESULT* phr, LPCWSTR pName, int nIndex)
: CBaseOutputPin(NAME("TM Multi Output Pin"), pFilter, pLock, phr, pName),
m_pFilter(pFilter),
m_nIndex(nIndex),
m_pOutputQueue(NULL)
{
}

CMultiIOOutputPin::~CMultiIOOutputPin(void)
{
}

HRESULT CMultiIOOutputPin::DecideBufferSize( IMemAllocator* pAlloc, ALLOCATOR_PROPERTIES* pRequestProperties )
{
	//Pass the buffer allocation on to the mixer filter
	return m_pFilter->DecideBufferSize(pAlloc, pRequestProperties, m_nIndex);
}

HRESULT CMultiIOOutputPin::CheckMediaType( const CMediaType* pMediaType )
{
	return m_pFilter->CheckOutputType(pMediaType);
}

HRESULT CMultiIOOutputPin::GetMediaType( int iPosition, CMediaType* pMediaType )
{
	ASSERT(m_pFilter->m_vInputPins.size() > 0);

	//  We don't have any media types if our input is not connected

	if (m_pFilter->m_vInputPins[0]->IsConnected()) 
	{
		return m_pFilter->GetMediaType(iPosition, pMediaType, m_nIndex);
	} else 
	{
		return VFW_S_NO_MORE_ITEMS;
	}
}

STDMETHODIMP CMultiIOOutputPin::Notify( IBaseFilter * pSender, Quality q )
{
	UNREFERENCED_PARAMETER(q);
	UNREFERENCED_PARAMETER(pSender);
	return E_NOTIMPL;
}

HRESULT CMultiIOOutputPin::Active()
{
	CAutoLock lock_it(m_pLock);
	HRESULT hr = NOERROR;

	// Make sure that the pin is connected
	if(m_Connected == NULL)
		return NOERROR;

	// Create the output queue if we have to
	if(m_pOutputQueue == NULL)
	{
		m_pOutputQueue = new COutputQueue(m_Connected, &hr, TRUE, FALSE);
		if(m_pOutputQueue == NULL)
			return E_OUTOFMEMORY;

		// Make sure that the constructor did not return any error
		if(FAILED(hr))
		{
			delete m_pOutputQueue;
			m_pOutputQueue = NULL;
			return hr;
		}
	}

	// Pass the call on to the base class
	CBaseOutputPin::Active();
	return NOERROR;

} // Active

HRESULT CMultiIOOutputPin::Inactive()
{
	CAutoLock lock_it(m_pLock);

	// Delete the output queus associated with the pin.
	if(m_pOutputQueue)
	{
		delete m_pOutputQueue;
		m_pOutputQueue = NULL;
	}

	CBaseOutputPin::Inactive();
	return NOERROR;

} // Inactive

HRESULT CMultiIOOutputPin::Deliver(IMediaSample *pMediaSample)
{
	CheckPointer(pMediaSample,E_POINTER);

	// Make sure that we have an output queue
	if(m_pOutputQueue == NULL)
		return NOERROR;

	pMediaSample->AddRef();
	return m_pOutputQueue->Receive(pMediaSample);

} // Deliver

//
// DeliverEndOfStream
//
HRESULT CMultiIOOutputPin::DeliverEndOfStream()
{
	// Make sure that we have an output queue
	if(m_pOutputQueue == NULL)
		return NOERROR;

	m_pOutputQueue->EOS();
	return NOERROR;

} // DeliverEndOfStream


//
// DeliverBeginFlush
//
HRESULT CMultiIOOutputPin::DeliverBeginFlush()
{
	// Make sure that we have an output queue
	if(m_pOutputQueue == NULL)
		return NOERROR;

	m_pOutputQueue->BeginFlush();
	return NOERROR;

} // DeliverBeginFlush


//
// DeliverEndFlush
//
HRESULT CMultiIOOutputPin::DeliverEndFlush()
{
	// Make sure that we have an output queue
	if(m_pOutputQueue == NULL)
		return NOERROR;

	m_pOutputQueue->EndFlush();
	return NOERROR;

} // DeliverEndFlish

//
// DeliverNewSegment
//
HRESULT CMultiIOOutputPin::DeliverNewSegment(REFERENCE_TIME tStart, 
										 REFERENCE_TIME tStop,  
										 double dRate)
{
	// Make sure that we have an output queue
	if(m_pOutputQueue == NULL)
		return NOERROR;

	m_pOutputQueue->NewSegment(tStart, tStop, dRate);
	return NOERROR;

} // DeliverNewSegment