/** @file
===========================================================================
MODULE          : directshowdevicesource

DESCRIPTIONS    : 

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#include "stdafx.h"
#include "DirectShowDeviceSource.h"

// For gettimeofday
#include "GroupsockHelper.hh"

#include <iostream>

CDirectShowDeviceSource* CDirectShowDeviceSource::createNew( UsageEnvironment& env/*, CDirectShowDeviceParameters params */)
{
	CDirectShowDeviceSource* pSource =  new CDirectShowDeviceSource(env/*, params*/);
	pSource->setScheduledTask();
	return pSource;
}

CDirectShowDeviceSource::CDirectShowDeviceSource( UsageEnvironment& env/*, CDirectShowDeviceParameters params */)
: FramedSource(env)
{
	// Do any initialization for the device here
}

CDirectShowDeviceSource::~CDirectShowDeviceSource()
{

}

void CDirectShowDeviceSource::doGetNextFrame()
{
	// Arrange here for our "deliverFrame" member function to be called
	// when the next frame of data becomes available from the device.
	// This must be done in a non-blocking fashion - i.e., so that we
	// return immediately from this function even if no data is
	// currently available.
	//
	// If the device can be implemented as a readable socket, then one easy
	// way to do this is using a call to
	//     envir().taskScheduler().turnOnBackgroundReadHandling( ... )
	// (See examples of this call in the "liveMedia" directory.)
	// If, for some reason, the source device stops being readable
	// (e.g., it gets closed), then you do the following:
	if (0 /* the source stops being readable */) {
		handleClosure(this);
		return;
	}
}

void CDirectShowDeviceSource::deliverFrame()
{
	// This would be called when new frame data is available from the device.
	// This function should deliver the next frame of data from the device,
	// using the following parameters (class members):
	// 'in' parameters (these should *not* be modified by this function):
	//     fTo: The frame data is copied to this address.
	//         (Note that the variable "fTo" is *not* modified.  Instead,
	//          the frame data is copied to the address pointed to by "fTo".)
	//     fMaxSize: This is the maximum number of bytes that can be copied
	//         (If the actual frame is larger than this, then it should
	//          be truncated, and "fNumTruncatedBytes" set accordingly.)
	// 'out' parameters (these are modified by this function):
	//     fFrameSize: Should be set to the delivered frame size (<= fMaxSize).
	//     fNumTruncatedBytes: Should be set iff the delivered frame would have been
	//         bigger than "fMaxSize", in which case it's set to the number of bytes
	//         that have been omitted.
	//     fPresentationTime: Should be set to the frame's presentation time
	//         (seconds, microseconds).
	//     fDurationInMicroseconds: Should be set to the frame's duration, if known.
	
	// Lock the queue since we have multiple threads accessing it
	////CMAutoLock* lock = new CMAutoLock(&m_Lock);
	CAutoLock lock(&m_Lock);

	if (!isCurrentlyAwaitingData()) return; // we're not ready for the data yet

	// Deliver the data here:
	if (m_qFrameQueue.size()==0)
	{
		// There's no media data available yet
		return;
	}
	else
	{
		//Get the first frame from the queue
		CMediaFrame* pMediaFrame = m_qFrameQueue.front();
		m_qFrameQueue.pop();

		if (pMediaFrame->GetActualDataLength() > fMaxSize)
		{
			//TOREVISE/TODO
			fNumTruncatedBytes = pMediaFrame->GetActualDataLength() -  fFrameSize;
			fFrameSize = fMaxSize;
			//TODO How do we send the rest in the following packet???
			//TODO How do we delete the frame??? Unless we store extra attributes in the MediaFrame class
		}
		else
		{
			fFrameSize = pMediaFrame->GetActualDataLength();
			memcpy(fTo, pMediaFrame->GetData(), fFrameSize);
			
			fPresentationTime.tv_sec = pMediaFrame->StartTime()/DIRECT_SHOW_TOLIVE_TIME_FACTOR/1000000; //seconds *10e-7 (0.0000001) for seconds
			fPresentationTime.tv_usec = (pMediaFrame->StartTime()/DIRECT_SHOW_TOLIVE_TIME_FACTOR)%1000000; //microseconds *10e-1 (0.1) for seconds
			fDurationInMicroseconds = pMediaFrame->Duration()/DIRECT_SHOW_TOLIVE_TIME_FACTOR;

			std::cerr << " Presentation Time: " << fPresentationTime.tv_sec << " " <<  fPresentationTime.tv_usec << std::endl;

			//Free the memory used by the frame
			delete pMediaFrame;
		}
	}

	// After delivering the data, inform the reader that it is now available:
	FramedSource::afterGetting(this);
}

void CDirectShowDeviceSource::doDeliverFrame( void* pInstance )
{
	CDirectShowDeviceSource* pSource = (CDirectShowDeviceSource*)pInstance;
	if (pSource)
	{
		pSource->deliverFrame();
	}
}

void CDirectShowDeviceSource::AddFrame( CMediaFrame* pMediaFrame )
{
	CAutoLock lock(&m_Lock);
	// Add the sample to the queue
	m_qFrameQueue.push(pMediaFrame);
}